/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_CORRIDOR_H
#define DPC_IMPL_TYPE_CORRIDOR_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint32.h"
#include "impl_type_uint8.h"
#include "impl_type_float.h"
#include "dpc/impl_type_linegeometry.h"
#include "dpc/impl_type_vector_objectlocation.h"

namespace dpc {
struct Corridor {
    ::UInt32 corridor_id{ 0U };
    ::UInt8 feature_tag{ 0U };
    ::UInt32 ilc_object_target{ 0U };
    ::UInt32 disenable_reason{ 0U };
    ::Float drivable_width{ 0.0F };
    ::dpc::LineGeometry reference_line;
    ::dpc::vector_ObjectLocation object_in_corridor_list;
    ::dpc::vector_ObjectLocation relevant_object_corridor_list;

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(corridor_id);
        fun(feature_tag);
        fun(ilc_object_target);
        fun(disenable_reason);
        fun(drivable_width);
        fun(reference_line);
        fun(object_in_corridor_list);
        fun(relevant_object_corridor_list);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(corridor_id);
        fun(feature_tag);
        fun(ilc_object_target);
        fun(disenable_reason);
        fun(drivable_width);
        fun(reference_line);
        fun(object_in_corridor_list);
        fun(relevant_object_corridor_list);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("corridor_id", corridor_id);
        fun("feature_tag", feature_tag);
        fun("ilc_object_target", ilc_object_target);
        fun("disenable_reason", disenable_reason);
        fun("drivable_width", drivable_width);
        fun("reference_line", reference_line);
        fun("object_in_corridor_list", object_in_corridor_list);
        fun("relevant_object_corridor_list", relevant_object_corridor_list);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("corridor_id", corridor_id);
        fun("feature_tag", feature_tag);
        fun("ilc_object_target", ilc_object_target);
        fun("disenable_reason", disenable_reason);
        fun("drivable_width", drivable_width);
        fun("reference_line", reference_line);
        fun("object_in_corridor_list", object_in_corridor_list);
        fun("relevant_object_corridor_list", relevant_object_corridor_list);
    }

    friend bool operator==(const ::dpc::Corridor& lhs, const ::dpc::Corridor& rhs) noexcept
    {
        return (lhs.corridor_id == rhs.corridor_id) && (lhs.feature_tag == rhs.feature_tag) && (lhs.ilc_object_target == rhs.ilc_object_target) && (lhs.disenable_reason == rhs.disenable_reason) && (fabs(static_cast<double>(lhs.drivable_width - rhs.drivable_width)) < DBL_EPSILON) && (lhs.reference_line == rhs.reference_line) && (lhs.object_in_corridor_list == rhs.object_in_corridor_list) && (lhs.relevant_object_corridor_list == rhs.relevant_object_corridor_list);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_CORRIDOR_H
