/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_LANEMARKER_H
#define DPC_IMPL_TYPE_LANEMARKER_H

#include "impl_type_uint32.h"
#include "dpc/impl_type_vector_laneboundary.h"
#include "dpc/impl_type_vector_linecolor.h"
#include "dpc/impl_type_vector_linetype.h"
#include "impl_type_boolean.h"

namespace dpc {
struct LaneMarker {
    ::UInt32 id{ 0U };
    ::dpc::vector_LaneBoundary front_lane_boundary_list;
    ::dpc::vector_LaneBoundary back_lane_boundary_list;
    ::dpc::vector_LineColor line_color_attribute_list;
    ::dpc::vector_LineType line_type_attribute_list;
    ::Boolean is_start_out_of_range{ false };
    ::Boolean is_end_out_of_range{ false };

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(id);
        fun(front_lane_boundary_list);
        fun(back_lane_boundary_list);
        fun(line_color_attribute_list);
        fun(line_type_attribute_list);
        fun(is_start_out_of_range);
        fun(is_end_out_of_range);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(id);
        fun(front_lane_boundary_list);
        fun(back_lane_boundary_list);
        fun(line_color_attribute_list);
        fun(line_type_attribute_list);
        fun(is_start_out_of_range);
        fun(is_end_out_of_range);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("id", id);
        fun("front_lane_boundary_list", front_lane_boundary_list);
        fun("back_lane_boundary_list", back_lane_boundary_list);
        fun("line_color_attribute_list", line_color_attribute_list);
        fun("line_type_attribute_list", line_type_attribute_list);
        fun("is_start_out_of_range", is_start_out_of_range);
        fun("is_end_out_of_range", is_end_out_of_range);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("id", id);
        fun("front_lane_boundary_list", front_lane_boundary_list);
        fun("back_lane_boundary_list", back_lane_boundary_list);
        fun("line_color_attribute_list", line_color_attribute_list);
        fun("line_type_attribute_list", line_type_attribute_list);
        fun("is_start_out_of_range", is_start_out_of_range);
        fun("is_end_out_of_range", is_end_out_of_range);
    }

    friend bool operator==(const ::dpc::LaneMarker& lhs, const ::dpc::LaneMarker& rhs) noexcept
    {
        return (lhs.id == rhs.id) && (lhs.front_lane_boundary_list == rhs.front_lane_boundary_list) && (lhs.back_lane_boundary_list == rhs.back_lane_boundary_list) && (lhs.line_color_attribute_list == rhs.line_color_attribute_list) && (lhs.line_type_attribute_list == rhs.line_type_attribute_list) && (lhs.is_start_out_of_range == rhs.is_start_out_of_range) && (lhs.is_end_out_of_range == rhs.is_end_out_of_range);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_LANEMARKER_H
