/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_NEIGHBORLANE_H
#define DPC_IMPL_TYPE_NEIGHBORLANE_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"
#include "impl_type_uint8.h"

namespace dpc {
struct NeighborLane {
    ::Float start_range{ 0.0F };
    ::Float end_range{ 0.0F };
    ::UInt8 lane_id{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(start_range);
        fun(end_range);
        fun(lane_id);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(start_range);
        fun(end_range);
        fun(lane_id);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("start_range", start_range);
        fun("end_range", end_range);
        fun("lane_id", lane_id);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("start_range", start_range);
        fun("end_range", end_range);
        fun("lane_id", lane_id);
    }

    friend bool operator==(const ::dpc::NeighborLane& lhs, const ::dpc::NeighborLane& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.start_range - rhs.start_range)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.end_range - rhs.end_range)) < DBL_EPSILON) && (lhs.lane_id == rhs.lane_id);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_NEIGHBORLANE_H
