/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_RMEGORGTLNEGEOMETRY_H
#define DPC_IMPL_TYPE_RMEGORGTLNEGEOMETRY_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"
#include "impl_type_boolean.h"

namespace dpc {
struct RMEgoRgtLnegeometry {
    ::Float c0_rlne{ 0.0F };
    ::Float c1_rlne{ 0.0F };
    ::Float c2_rlne{ 0.0F };
    ::Float c3_rlne{ 0.0F };
    ::Float start_rnge_rlne{ 0.0F };
    ::Float end_rnge_rlne{ 0.0F };
    ::Float probability_rlne{ 0.0F };
    ::Boolean valid_rlne{ false };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(c0_rlne);
        fun(c1_rlne);
        fun(c2_rlne);
        fun(c3_rlne);
        fun(start_rnge_rlne);
        fun(end_rnge_rlne);
        fun(probability_rlne);
        fun(valid_rlne);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(c0_rlne);
        fun(c1_rlne);
        fun(c2_rlne);
        fun(c3_rlne);
        fun(start_rnge_rlne);
        fun(end_rnge_rlne);
        fun(probability_rlne);
        fun(valid_rlne);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("c0_rlne", c0_rlne);
        fun("c1_rlne", c1_rlne);
        fun("c2_rlne", c2_rlne);
        fun("c3_rlne", c3_rlne);
        fun("start_rnge_rlne", start_rnge_rlne);
        fun("end_rnge_rlne", end_rnge_rlne);
        fun("probability_rlne", probability_rlne);
        fun("valid_rlne", valid_rlne);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("c0_rlne", c0_rlne);
        fun("c1_rlne", c1_rlne);
        fun("c2_rlne", c2_rlne);
        fun("c3_rlne", c3_rlne);
        fun("start_rnge_rlne", start_rnge_rlne);
        fun("end_rnge_rlne", end_rnge_rlne);
        fun("probability_rlne", probability_rlne);
        fun("valid_rlne", valid_rlne);
    }

    friend bool operator==(const ::dpc::RMEgoRgtLnegeometry& lhs, const ::dpc::RMEgoRgtLnegeometry& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.c0_rlne - rhs.c0_rlne)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.c1_rlne - rhs.c1_rlne)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.c2_rlne - rhs.c2_rlne)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.c3_rlne - rhs.c3_rlne)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.start_rnge_rlne - rhs.start_rnge_rlne)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.end_rnge_rlne - rhs.end_rnge_rlne)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.probability_rlne - rhs.probability_rlne)) < DBL_EPSILON) && (lhs.valid_rlne == rhs.valid_rlne);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_RMEGORGTLNEGEOMETRY_H
