/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_RMHEADER_H
#define DPC_IMPL_TYPE_RMHEADER_H

#include "impl_type_uint32.h"
#include "impl_type_uint64.h"
#include "impl_type_uint16.h"

namespace dpc {
struct RMHeader {
    ::UInt32 num_sequence{ 0U };
    ::UInt64 timestamp{ 0U };
    ::UInt32 data_version{ 0U };
    ::UInt32 software_version{ 0U };
    ::UInt32 status{ 0U };
    ::UInt16 cyclecounter{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(num_sequence);
        fun(timestamp);
        fun(data_version);
        fun(software_version);
        fun(status);
        fun(cyclecounter);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(num_sequence);
        fun(timestamp);
        fun(data_version);
        fun(software_version);
        fun(status);
        fun(cyclecounter);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("num_sequence", num_sequence);
        fun("timestamp", timestamp);
        fun("data_version", data_version);
        fun("software_version", software_version);
        fun("status", status);
        fun("cyclecounter", cyclecounter);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("num_sequence", num_sequence);
        fun("timestamp", timestamp);
        fun("data_version", data_version);
        fun("software_version", software_version);
        fun("status", status);
        fun("cyclecounter", cyclecounter);
    }

    friend bool operator==(const ::dpc::RMHeader& lhs, const ::dpc::RMHeader& rhs) noexcept
    {
        return (lhs.num_sequence == rhs.num_sequence) && (lhs.timestamp == rhs.timestamp) && (lhs.data_version == rhs.data_version) && (lhs.software_version == rhs.software_version) && (lhs.status == rhs.status) && (lhs.cyclecounter == rhs.cyclecounter);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_RMHEADER_H
