/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_ROAD_EDGE_H
#define DPC_IMPL_TYPE_ROAD_EDGE_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint8.h"
#include "dpc/impl_type_array_uint8_t_2.h"
#include "impl_type_float.h"
#include "dpc/impl_type_vector_position.h"

namespace dpc {
struct road_edge {
    ::UInt8 type{ 0U };
    ::UInt8 side{ 0U };
    ::dpc::array_uint8_t_2 padding{};
    ::Float start_offset{ 0.0F };
    ::Float end_offset{ 0.0F };
    ::dpc::vector_position points;

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(type);
        fun(side);
        fun(padding);
        fun(start_offset);
        fun(end_offset);
        fun(points);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(type);
        fun(side);
        fun(padding);
        fun(start_offset);
        fun(end_offset);
        fun(points);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("type", type);
        fun("side", side);
        fun("padding", padding);
        fun("start_offset", start_offset);
        fun("end_offset", end_offset);
        fun("points", points);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("type", type);
        fun("side", side);
        fun("padding", padding);
        fun("start_offset", start_offset);
        fun("end_offset", end_offset);
        fun("points", points);
    }

    friend bool operator==(const ::dpc::road_edge& lhs, const ::dpc::road_edge& rhs) noexcept
    {
        return (lhs.type == rhs.type) && (lhs.side == rhs.side) && (lhs.padding == rhs.padding) && (fabs(static_cast<double>(lhs.start_offset - rhs.start_offset)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.end_offset - rhs.end_offset)) < DBL_EPSILON) && (lhs.points == rhs.points);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_ROAD_EDGE_H
