/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_SLAM_ODOMETRY_H
#define DPC_IMPL_TYPE_SLAM_ODOMETRY_H

#include "impl_type_uint64.h"
#include "dpc/impl_type_pose.h"
#include "impl_type_uint32.h"
#include "impl_type_uint8.h"
#include "dpc/impl_type_array_uint8_t_3.h"

namespace dpc {
struct slam_odometry {
    ::UInt64 utc_timestamp{ 0U };
    ::dpc::Pose position;
    ::UInt32 cycle_counter{ 0U };
    ::UInt8 state{ 0U };
    ::dpc::array_uint8_t_3 padding{};

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(utc_timestamp);
        fun(position);
        fun(cycle_counter);
        fun(state);
        fun(padding);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(utc_timestamp);
        fun(position);
        fun(cycle_counter);
        fun(state);
        fun(padding);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("utc_timestamp", utc_timestamp);
        fun("position", position);
        fun("cycle_counter", cycle_counter);
        fun("state", state);
        fun("padding", padding);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("utc_timestamp", utc_timestamp);
        fun("position", position);
        fun("cycle_counter", cycle_counter);
        fun("state", state);
        fun("padding", padding);
    }

    friend bool operator==(const ::dpc::slam_odometry& lhs, const ::dpc::slam_odometry& rhs) noexcept
    {
        return (lhs.utc_timestamp == rhs.utc_timestamp) && (lhs.position == rhs.position) && (lhs.cycle_counter == rhs.cycle_counter) && (lhs.state == rhs.state) && (lhs.padding == rhs.padding);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_SLAM_ODOMETRY_H
