/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_SLAM_ROAD_MODEL_H
#define DPC_IMPL_TYPE_SLAM_ROAD_MODEL_H

#include "impl_type_uint64.h"
#include "impl_type_uint32.h"
#include "dpc/impl_type_vector_traffic_lane.h"
#include "dpc/impl_type_vector_road_edge.h"
#include "dpc/impl_type_vector_line.h"

namespace dpc {
struct slam_road_model {
    ::UInt64 utc_timestamp{ 0U };
    ::UInt32 cycle_counter{ 0U };
    ::dpc::vector_traffic_lane lanes;
    ::dpc::vector_road_edge road_edges;
    ::dpc::vector_line lines;

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(utc_timestamp);
        fun(cycle_counter);
        fun(lanes);
        fun(road_edges);
        fun(lines);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(utc_timestamp);
        fun(cycle_counter);
        fun(lanes);
        fun(road_edges);
        fun(lines);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("utc_timestamp", utc_timestamp);
        fun("cycle_counter", cycle_counter);
        fun("lanes", lanes);
        fun("road_edges", road_edges);
        fun("lines", lines);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("utc_timestamp", utc_timestamp);
        fun("cycle_counter", cycle_counter);
        fun("lanes", lanes);
        fun("road_edges", road_edges);
        fun("lines", lines);
    }

    friend bool operator==(const ::dpc::slam_road_model& lhs, const ::dpc::slam_road_model& rhs) noexcept
    {
        return (lhs.utc_timestamp == rhs.utc_timestamp) && (lhs.cycle_counter == rhs.cycle_counter) && (lhs.lanes == rhs.lanes) && (lhs.road_edges == rhs.road_edges) && (lhs.lines == rhs.lines);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_SLAM_ROAD_MODEL_H
