/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_SPEED_LIMIT_H
#define DPC_IMPL_TYPE_SPEED_LIMIT_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"

namespace dpc {
struct speed_limit {
    ::Float max_speed_limit{ 0.0F };
    ::Float min_speed_limit{ 0.0F };
    ::Float rec_speed_limit{ 0.0F };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(max_speed_limit);
        fun(min_speed_limit);
        fun(rec_speed_limit);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(max_speed_limit);
        fun(min_speed_limit);
        fun(rec_speed_limit);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("max_speed_limit", max_speed_limit);
        fun("min_speed_limit", min_speed_limit);
        fun("rec_speed_limit", rec_speed_limit);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("max_speed_limit", max_speed_limit);
        fun("min_speed_limit", min_speed_limit);
        fun("rec_speed_limit", rec_speed_limit);
    }

    friend bool operator==(const ::dpc::speed_limit& lhs, const ::dpc::speed_limit& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.max_speed_limit - rhs.max_speed_limit)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.min_speed_limit - rhs.min_speed_limit)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.rec_speed_limit - rhs.rec_speed_limit)) < DBL_EPSILON);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_SPEED_LIMIT_H
