/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_TRAFFIC_LANE_H
#define DPC_IMPL_TYPE_TRAFFIC_LANE_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint8.h"
#include "impl_type_uint32.h"
#include "impl_type_float.h"
#include "dpc/impl_type_speed_limit.h"
#include "dpc/impl_type_vector_traffic_object.h"

namespace dpc {
struct traffic_lane {
    ::UInt8 lane_type{ 0U };
    ::UInt8 lane_no{ 0U };
    ::UInt32 lane_id{ 0U };
    ::UInt32 center_line_id{ 0U };
    ::UInt32 left_line_id{ 0U };
    ::UInt32 right_line_id{ 0U };
    ::Float lane_width{ 0.0F };
    ::Float start_offset{ 0.0F };
    ::Float end_offset{ 0.0F };
    ::dpc::speed_limit speed_limit_value;
    ::dpc::vector_traffic_object traffic_objects;

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(lane_type);
        fun(lane_no);
        fun(lane_id);
        fun(center_line_id);
        fun(left_line_id);
        fun(right_line_id);
        fun(lane_width);
        fun(start_offset);
        fun(end_offset);
        fun(speed_limit_value);
        fun(traffic_objects);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(lane_type);
        fun(lane_no);
        fun(lane_id);
        fun(center_line_id);
        fun(left_line_id);
        fun(right_line_id);
        fun(lane_width);
        fun(start_offset);
        fun(end_offset);
        fun(speed_limit_value);
        fun(traffic_objects);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("lane_type", lane_type);
        fun("lane_no", lane_no);
        fun("lane_id", lane_id);
        fun("center_line_id", center_line_id);
        fun("left_line_id", left_line_id);
        fun("right_line_id", right_line_id);
        fun("lane_width", lane_width);
        fun("start_offset", start_offset);
        fun("end_offset", end_offset);
        fun("speed_limit_value", speed_limit_value);
        fun("traffic_objects", traffic_objects);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("lane_type", lane_type);
        fun("lane_no", lane_no);
        fun("lane_id", lane_id);
        fun("center_line_id", center_line_id);
        fun("left_line_id", left_line_id);
        fun("right_line_id", right_line_id);
        fun("lane_width", lane_width);
        fun("start_offset", start_offset);
        fun("end_offset", end_offset);
        fun("speed_limit_value", speed_limit_value);
        fun("traffic_objects", traffic_objects);
    }

    friend bool operator==(const ::dpc::traffic_lane& lhs, const ::dpc::traffic_lane& rhs) noexcept
    {
        return (lhs.lane_type == rhs.lane_type) && (lhs.lane_no == rhs.lane_no) && (lhs.lane_id == rhs.lane_id) && (lhs.center_line_id == rhs.center_line_id) && (lhs.left_line_id == rhs.left_line_id) && (lhs.right_line_id == rhs.right_line_id) && (fabs(static_cast<double>(lhs.lane_width - rhs.lane_width)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.start_offset - rhs.start_offset)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.end_offset - rhs.end_offset)) < DBL_EPSILON) && (lhs.speed_limit_value == rhs.speed_limit_value) && (lhs.traffic_objects == rhs.traffic_objects);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_TRAFFIC_LANE_H
