/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_TRAFFIC_OBJECT_H
#define DPC_IMPL_TYPE_TRAFFIC_OBJECT_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint32.h"
#include "impl_type_uint8.h"
#include "dpc/impl_type_array_uint8_t_2.h"
#include "impl_type_float.h"

namespace dpc {
struct traffic_object {
    ::UInt32 object_id{ 0U };
    ::UInt8 object_type{ 0U };
    ::UInt8 object_state{ 0U };
    ::dpc::array_uint8_t_2 padding{};
    ::Float start_offset{ 0.0F };
    ::Float end_offset{ 0.0F };

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(object_id);
        fun(object_type);
        fun(object_state);
        fun(padding);
        fun(start_offset);
        fun(end_offset);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(object_id);
        fun(object_type);
        fun(object_state);
        fun(padding);
        fun(start_offset);
        fun(end_offset);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("object_id", object_id);
        fun("object_type", object_type);
        fun("object_state", object_state);
        fun("padding", padding);
        fun("start_offset", start_offset);
        fun("end_offset", end_offset);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("object_id", object_id);
        fun("object_type", object_type);
        fun("object_state", object_state);
        fun("padding", padding);
        fun("start_offset", start_offset);
        fun("end_offset", end_offset);
    }

    friend bool operator==(const ::dpc::traffic_object& lhs, const ::dpc::traffic_object& rhs) noexcept
    {
        return (lhs.object_id == rhs.object_id) && (lhs.object_type == rhs.object_type) && (lhs.object_state == rhs.object_state) && (lhs.padding == rhs.padding) && (fabs(static_cast<double>(lhs.start_offset - rhs.start_offset)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.end_offset - rhs.end_offset)) < DBL_EPSILON);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_TRAFFIC_OBJECT_H
