/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef IMPL_TYPE_LIDAROBJECTS_H
#define IMPL_TYPE_LIDAROBJECTS_H

#include <cfloat>
#include <cmath>
#include "impl_type_int32.h"
#include "impl_type_point3d.h"
#include "impl_type_double.h"
#include "impl_type_vector_point3d.h"
#include "impl_type_uint8.h"
#include "impl_type_sensormeasurement.h"
#include "impl_type_uint83by3matrix.h"
#include "impl_type_double3by3matrix.h"

struct LidarObjects {
    ::Int32 ID{ 0 };
    ::Point3D position;
    ::Double theta{ 0.0 };
    ::Double length{ 0.0 };
    ::Double width{ 0.0 };
    ::Double height{ 0.0 };
    ::Point3D velocity;
    ::vector_Point3D polygon_point;
    ::UInt8 type{ 0U };
    ::UInt8 subtype{ 0U };
    ::Double timestamp{ 0.0 };
    ::vector_Point3D point_cloud;
    ::Double confidence{ 0.0 };
    ::SensorMeasurement measurements;
    ::Point3D acceleration;
    ::UInt83by3Matrix position_covariance{};
    ::Double3by3Matrix velocity_covariance{};
    ::Double3by3Matrix acceleration_covariance{};

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(ID);
        fun(position);
        fun(theta);
        fun(length);
        fun(width);
        fun(height);
        fun(velocity);
        fun(polygon_point);
        fun(type);
        fun(subtype);
        fun(timestamp);
        fun(point_cloud);
        fun(confidence);
        fun(measurements);
        fun(acceleration);
        fun(position_covariance);
        fun(velocity_covariance);
        fun(acceleration_covariance);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(ID);
        fun(position);
        fun(theta);
        fun(length);
        fun(width);
        fun(height);
        fun(velocity);
        fun(polygon_point);
        fun(type);
        fun(subtype);
        fun(timestamp);
        fun(point_cloud);
        fun(confidence);
        fun(measurements);
        fun(acceleration);
        fun(position_covariance);
        fun(velocity_covariance);
        fun(acceleration_covariance);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("ID", ID);
        fun("position", position);
        fun("theta", theta);
        fun("length", length);
        fun("width", width);
        fun("height", height);
        fun("velocity", velocity);
        fun("polygon_point", polygon_point);
        fun("type", type);
        fun("subtype", subtype);
        fun("timestamp", timestamp);
        fun("point_cloud", point_cloud);
        fun("confidence", confidence);
        fun("measurements", measurements);
        fun("acceleration", acceleration);
        fun("position_covariance", position_covariance);
        fun("velocity_covariance", velocity_covariance);
        fun("acceleration_covariance", acceleration_covariance);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("ID", ID);
        fun("position", position);
        fun("theta", theta);
        fun("length", length);
        fun("width", width);
        fun("height", height);
        fun("velocity", velocity);
        fun("polygon_point", polygon_point);
        fun("type", type);
        fun("subtype", subtype);
        fun("timestamp", timestamp);
        fun("point_cloud", point_cloud);
        fun("confidence", confidence);
        fun("measurements", measurements);
        fun("acceleration", acceleration);
        fun("position_covariance", position_covariance);
        fun("velocity_covariance", velocity_covariance);
        fun("acceleration_covariance", acceleration_covariance);
    }

    friend bool operator==(const ::LidarObjects& lhs, const ::LidarObjects& rhs) noexcept
    {
        return (lhs.ID == rhs.ID) && (lhs.position == rhs.position) && (fabs(static_cast<double>(lhs.theta - rhs.theta)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.length - rhs.length)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.width - rhs.width)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.height - rhs.height)) < DBL_EPSILON) && (lhs.velocity == rhs.velocity) && (lhs.polygon_point == rhs.polygon_point) && (lhs.type == rhs.type) && (lhs.subtype == rhs.subtype) && (fabs(static_cast<double>(lhs.timestamp - rhs.timestamp)) < DBL_EPSILON) && (lhs.point_cloud == rhs.point_cloud) && (fabs(static_cast<double>(lhs.confidence - rhs.confidence)) < DBL_EPSILON) && (lhs.measurements == rhs.measurements) && (lhs.acceleration == rhs.acceleration) && (lhs.position_covariance == rhs.position_covariance) && (lhs.velocity_covariance == rhs.velocity_covariance) && (lhs.acceleration_covariance == rhs.acceleration_covariance);
    }
};


#endif // IMPL_TYPE_LIDAROBJECTS_H
