/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef IMPL_TYPE_PHYSICAL_CONTROL_REQUEST1_T_H
#define IMPL_TYPE_PHYSICAL_CONTROL_REQUEST1_T_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint8.h"
#include "impl_type_double.h"
#include "impl_type_uint64.h"

struct physical_control_request1_t {
    ::UInt8 ipc_target_motor_en{ 0U };
    ::UInt8 ipc_target_gear{ 0U };
    ::UInt8 ipc_target_mode{ 0U };
    ::UInt8 ipc_life_signal{ 0U };
    ::Double ipc_target_speed{ 0.0 };
    ::UInt8 ipc_target_spd_valid{ 0U };
    ::Double ipc_target_brk_press{ 0.0 };
    ::Double ipc_target_angle{ 0.0 };
    ::UInt8 ipc_target_steering_en{ 0U };
    ::Double ipc_target_torque{ 0.0 };
    ::UInt8 ipc_torque_rpm_mode{ 0U };
    ::UInt8 ipc_turn_light{ 0U };
    ::UInt8 ipc_target_spd_mode{ 0U };
    ::UInt64 timestamp_s{ 0U };
    ::UInt64 timestamp_ns{ 0U };
    ::UInt8 crc8{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(ipc_target_motor_en);
        fun(ipc_target_gear);
        fun(ipc_target_mode);
        fun(ipc_life_signal);
        fun(ipc_target_speed);
        fun(ipc_target_spd_valid);
        fun(ipc_target_brk_press);
        fun(ipc_target_angle);
        fun(ipc_target_steering_en);
        fun(ipc_target_torque);
        fun(ipc_torque_rpm_mode);
        fun(ipc_turn_light);
        fun(ipc_target_spd_mode);
        fun(timestamp_s);
        fun(timestamp_ns);
        fun(crc8);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(ipc_target_motor_en);
        fun(ipc_target_gear);
        fun(ipc_target_mode);
        fun(ipc_life_signal);
        fun(ipc_target_speed);
        fun(ipc_target_spd_valid);
        fun(ipc_target_brk_press);
        fun(ipc_target_angle);
        fun(ipc_target_steering_en);
        fun(ipc_target_torque);
        fun(ipc_torque_rpm_mode);
        fun(ipc_turn_light);
        fun(ipc_target_spd_mode);
        fun(timestamp_s);
        fun(timestamp_ns);
        fun(crc8);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("ipc_target_motor_en", ipc_target_motor_en);
        fun("ipc_target_gear", ipc_target_gear);
        fun("ipc_target_mode", ipc_target_mode);
        fun("ipc_life_signal", ipc_life_signal);
        fun("ipc_target_speed", ipc_target_speed);
        fun("ipc_target_spd_valid", ipc_target_spd_valid);
        fun("ipc_target_brk_press", ipc_target_brk_press);
        fun("ipc_target_angle", ipc_target_angle);
        fun("ipc_target_steering_en", ipc_target_steering_en);
        fun("ipc_target_torque", ipc_target_torque);
        fun("ipc_torque_rpm_mode", ipc_torque_rpm_mode);
        fun("ipc_turn_light", ipc_turn_light);
        fun("ipc_target_spd_mode", ipc_target_spd_mode);
        fun("timestamp_s", timestamp_s);
        fun("timestamp_ns", timestamp_ns);
        fun("crc8", crc8);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("ipc_target_motor_en", ipc_target_motor_en);
        fun("ipc_target_gear", ipc_target_gear);
        fun("ipc_target_mode", ipc_target_mode);
        fun("ipc_life_signal", ipc_life_signal);
        fun("ipc_target_speed", ipc_target_speed);
        fun("ipc_target_spd_valid", ipc_target_spd_valid);
        fun("ipc_target_brk_press", ipc_target_brk_press);
        fun("ipc_target_angle", ipc_target_angle);
        fun("ipc_target_steering_en", ipc_target_steering_en);
        fun("ipc_target_torque", ipc_target_torque);
        fun("ipc_torque_rpm_mode", ipc_torque_rpm_mode);
        fun("ipc_turn_light", ipc_turn_light);
        fun("ipc_target_spd_mode", ipc_target_spd_mode);
        fun("timestamp_s", timestamp_s);
        fun("timestamp_ns", timestamp_ns);
        fun("crc8", crc8);
    }

    friend bool operator==(const ::physical_control_request1_t& lhs, const ::physical_control_request1_t& rhs) noexcept
    {
        return (lhs.ipc_target_motor_en == rhs.ipc_target_motor_en) && (lhs.ipc_target_gear == rhs.ipc_target_gear) && (lhs.ipc_target_mode == rhs.ipc_target_mode) && (lhs.ipc_life_signal == rhs.ipc_life_signal) && (fabs(static_cast<double>(lhs.ipc_target_speed - rhs.ipc_target_speed)) < DBL_EPSILON) && (lhs.ipc_target_spd_valid == rhs.ipc_target_spd_valid) && (fabs(static_cast<double>(lhs.ipc_target_brk_press - rhs.ipc_target_brk_press)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.ipc_target_angle - rhs.ipc_target_angle)) < DBL_EPSILON) && (lhs.ipc_target_steering_en == rhs.ipc_target_steering_en) && (fabs(static_cast<double>(lhs.ipc_target_torque - rhs.ipc_target_torque)) < DBL_EPSILON) && (lhs.ipc_torque_rpm_mode == rhs.ipc_torque_rpm_mode) && (lhs.ipc_turn_light == rhs.ipc_turn_light) && (lhs.ipc_target_spd_mode == rhs.ipc_target_spd_mode) && (lhs.timestamp_s == rhs.timestamp_s) && (lhs.timestamp_ns == rhs.timestamp_ns) && (lhs.crc8 == rhs.crc8);
    }
};


#endif // IMPL_TYPE_PHYSICAL_CONTROL_REQUEST1_T_H
