/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef IMPL_TYPE_PHYSICAL_GNSS_SOL_RESULT_SPD_T_H
#define IMPL_TYPE_PHYSICAL_GNSS_SOL_RESULT_SPD_T_H

#include <cfloat>
#include <cmath>
#include "impl_type_double.h"
#include "impl_type_uint64.h"
#include "impl_type_uint8.h"

struct physical_gnss_sol_result_spd_t {
    ::Double gnss_horizontal_spd{ 0.0 };
    ::Double gnss_track_over_ground{ 0.0 };
    ::Double gnss_vertical_spd{ 0.0 };
    ::UInt64 timestamp_s{ 0U };
    ::UInt64 timestamp_ns{ 0U };
    ::UInt8 crc8{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(gnss_horizontal_spd);
        fun(gnss_track_over_ground);
        fun(gnss_vertical_spd);
        fun(timestamp_s);
        fun(timestamp_ns);
        fun(crc8);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(gnss_horizontal_spd);
        fun(gnss_track_over_ground);
        fun(gnss_vertical_spd);
        fun(timestamp_s);
        fun(timestamp_ns);
        fun(crc8);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("gnss_horizontal_spd", gnss_horizontal_spd);
        fun("gnss_track_over_ground", gnss_track_over_ground);
        fun("gnss_vertical_spd", gnss_vertical_spd);
        fun("timestamp_s", timestamp_s);
        fun("timestamp_ns", timestamp_ns);
        fun("crc8", crc8);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("gnss_horizontal_spd", gnss_horizontal_spd);
        fun("gnss_track_over_ground", gnss_track_over_ground);
        fun("gnss_vertical_spd", gnss_vertical_spd);
        fun("timestamp_s", timestamp_s);
        fun("timestamp_ns", timestamp_ns);
        fun("crc8", crc8);
    }

    friend bool operator==(const ::physical_gnss_sol_result_spd_t& lhs, const ::physical_gnss_sol_result_spd_t& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.gnss_horizontal_spd - rhs.gnss_horizontal_spd)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.gnss_track_over_ground - rhs.gnss_track_over_ground)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.gnss_vertical_spd - rhs.gnss_vertical_spd)) < DBL_EPSILON) && (lhs.timestamp_s == rhs.timestamp_s) && (lhs.timestamp_ns == rhs.timestamp_ns) && (lhs.crc8 == rhs.crc8);
    }
};


#endif // IMPL_TYPE_PHYSICAL_GNSS_SOL_RESULT_SPD_T_H
