/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef IMPL_TYPE_PHYSICAL_HEADING_ANGLE_T_H
#define IMPL_TYPE_PHYSICAL_HEADING_ANGLE_T_H

#include <cfloat>
#include <cmath>
#include "impl_type_double.h"
#include "impl_type_uint64.h"
#include "impl_type_uint8.h"

struct physical_heading_angle_t {
    ::Double heading_heading{ 0.0 };
    ::Double heading_pitch{ 0.0 };
    ::Double heading_heading_std{ 0.0 };
    ::Double heading_pitch_std{ 0.0 };
    ::Double heading_length{ 0.0 };
    ::UInt64 timestamp_s{ 0U };
    ::UInt64 timestamp_ns{ 0U };
    ::UInt8 crc8{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(heading_heading);
        fun(heading_pitch);
        fun(heading_heading_std);
        fun(heading_pitch_std);
        fun(heading_length);
        fun(timestamp_s);
        fun(timestamp_ns);
        fun(crc8);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(heading_heading);
        fun(heading_pitch);
        fun(heading_heading_std);
        fun(heading_pitch_std);
        fun(heading_length);
        fun(timestamp_s);
        fun(timestamp_ns);
        fun(crc8);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("heading_heading", heading_heading);
        fun("heading_pitch", heading_pitch);
        fun("heading_heading_std", heading_heading_std);
        fun("heading_pitch_std", heading_pitch_std);
        fun("heading_length", heading_length);
        fun("timestamp_s", timestamp_s);
        fun("timestamp_ns", timestamp_ns);
        fun("crc8", crc8);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("heading_heading", heading_heading);
        fun("heading_pitch", heading_pitch);
        fun("heading_heading_std", heading_heading_std);
        fun("heading_pitch_std", heading_pitch_std);
        fun("heading_length", heading_length);
        fun("timestamp_s", timestamp_s);
        fun("timestamp_ns", timestamp_ns);
        fun("crc8", crc8);
    }

    friend bool operator==(const ::physical_heading_angle_t& lhs, const ::physical_heading_angle_t& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.heading_heading - rhs.heading_heading)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.heading_pitch - rhs.heading_pitch)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.heading_heading_std - rhs.heading_heading_std)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.heading_pitch_std - rhs.heading_pitch_std)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.heading_length - rhs.heading_length)) < DBL_EPSILON) && (lhs.timestamp_s == rhs.timestamp_s) && (lhs.timestamp_ns == rhs.timestamp_ns) && (lhs.crc8 == rhs.crc8);
    }
};


#endif // IMPL_TYPE_PHYSICAL_HEADING_ANGLE_T_H
