/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef IMPL_TYPE_PHYSICAL_IMU_BIAS_CORRECTED_ACC_T_H
#define IMPL_TYPE_PHYSICAL_IMU_BIAS_CORRECTED_ACC_T_H

#include <cfloat>
#include <cmath>
#include "impl_type_double.h"
#include "impl_type_uint64.h"
#include "impl_type_uint8.h"

struct physical_imu_bias_corrected_acc_t {
    ::Double imu_bias_corrected_acc_x{ 0.0 };
    ::Double imu_bias_corrected_acc_y{ 0.0 };
    ::Double imu_bias_corrected_acc_z{ 0.0 };
    ::UInt64 timestamp_s{ 0U };
    ::UInt64 timestamp_ns{ 0U };
    ::UInt8 crc8{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(imu_bias_corrected_acc_x);
        fun(imu_bias_corrected_acc_y);
        fun(imu_bias_corrected_acc_z);
        fun(timestamp_s);
        fun(timestamp_ns);
        fun(crc8);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(imu_bias_corrected_acc_x);
        fun(imu_bias_corrected_acc_y);
        fun(imu_bias_corrected_acc_z);
        fun(timestamp_s);
        fun(timestamp_ns);
        fun(crc8);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("imu_bias_corrected_acc_x", imu_bias_corrected_acc_x);
        fun("imu_bias_corrected_acc_y", imu_bias_corrected_acc_y);
        fun("imu_bias_corrected_acc_z", imu_bias_corrected_acc_z);
        fun("timestamp_s", timestamp_s);
        fun("timestamp_ns", timestamp_ns);
        fun("crc8", crc8);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("imu_bias_corrected_acc_x", imu_bias_corrected_acc_x);
        fun("imu_bias_corrected_acc_y", imu_bias_corrected_acc_y);
        fun("imu_bias_corrected_acc_z", imu_bias_corrected_acc_z);
        fun("timestamp_s", timestamp_s);
        fun("timestamp_ns", timestamp_ns);
        fun("crc8", crc8);
    }

    friend bool operator==(const ::physical_imu_bias_corrected_acc_t& lhs, const ::physical_imu_bias_corrected_acc_t& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.imu_bias_corrected_acc_x - rhs.imu_bias_corrected_acc_x)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.imu_bias_corrected_acc_y - rhs.imu_bias_corrected_acc_y)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.imu_bias_corrected_acc_z - rhs.imu_bias_corrected_acc_z)) < DBL_EPSILON) && (lhs.timestamp_s == rhs.timestamp_s) && (lhs.timestamp_ns == rhs.timestamp_ns) && (lhs.crc8 == rhs.crc8);
    }
};


#endif // IMPL_TYPE_PHYSICAL_IMU_BIAS_CORRECTED_ACC_T_H
