/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef IMPL_TYPE_PHYSICAL_IMU_ORIGIN_COMMON_T_H
#define IMPL_TYPE_PHYSICAL_IMU_ORIGIN_COMMON_T_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint16.h"
#include "impl_type_uint32.h"
#include "impl_type_double.h"
#include "impl_type_uint64.h"
#include "impl_type_uint8.h"

struct physical_imu_origin_common_t {
    ::UInt16 imu_ori_gps_week{ 0U };
    ::UInt32 imu_ori_gps_sow{ 0U };
    ::Double imu_temperatue{ 0.0 };
    ::UInt64 timestamp_s{ 0U };
    ::UInt64 timestamp_ns{ 0U };
    ::UInt8 crc8{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(imu_ori_gps_week);
        fun(imu_ori_gps_sow);
        fun(imu_temperatue);
        fun(timestamp_s);
        fun(timestamp_ns);
        fun(crc8);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(imu_ori_gps_week);
        fun(imu_ori_gps_sow);
        fun(imu_temperatue);
        fun(timestamp_s);
        fun(timestamp_ns);
        fun(crc8);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("imu_ori_gps_week", imu_ori_gps_week);
        fun("imu_ori_gps_sow", imu_ori_gps_sow);
        fun("imu_temperatue", imu_temperatue);
        fun("timestamp_s", timestamp_s);
        fun("timestamp_ns", timestamp_ns);
        fun("crc8", crc8);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("imu_ori_gps_week", imu_ori_gps_week);
        fun("imu_ori_gps_sow", imu_ori_gps_sow);
        fun("imu_temperatue", imu_temperatue);
        fun("timestamp_s", timestamp_s);
        fun("timestamp_ns", timestamp_ns);
        fun("crc8", crc8);
    }

    friend bool operator==(const ::physical_imu_origin_common_t& lhs, const ::physical_imu_origin_common_t& rhs) noexcept
    {
        return (lhs.imu_ori_gps_week == rhs.imu_ori_gps_week) && (lhs.imu_ori_gps_sow == rhs.imu_ori_gps_sow) && (fabs(static_cast<double>(lhs.imu_temperatue - rhs.imu_temperatue)) < DBL_EPSILON) && (lhs.timestamp_s == rhs.timestamp_s) && (lhs.timestamp_ns == rhs.timestamp_ns) && (lhs.crc8 == rhs.crc8);
    }
};


#endif // IMPL_TYPE_PHYSICAL_IMU_ORIGIN_COMMON_T_H
