/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef IMPL_TYPE_PHYSICAL_IMU_ORIGIN_GYRO_T_H
#define IMPL_TYPE_PHYSICAL_IMU_ORIGIN_GYRO_T_H

#include <cfloat>
#include <cmath>
#include "impl_type_double.h"
#include "impl_type_uint64.h"
#include "impl_type_uint8.h"

struct physical_imu_origin_gyro_t {
    ::Double imu_ori_gyro_x{ 0.0 };
    ::Double imu_ori_gyro_y{ 0.0 };
    ::Double imu_ori_gyro_z{ 0.0 };
    ::UInt64 timestamp_s{ 0U };
    ::UInt64 timestamp_ns{ 0U };
    ::UInt8 crc8{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(imu_ori_gyro_x);
        fun(imu_ori_gyro_y);
        fun(imu_ori_gyro_z);
        fun(timestamp_s);
        fun(timestamp_ns);
        fun(crc8);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(imu_ori_gyro_x);
        fun(imu_ori_gyro_y);
        fun(imu_ori_gyro_z);
        fun(timestamp_s);
        fun(timestamp_ns);
        fun(crc8);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("imu_ori_gyro_x", imu_ori_gyro_x);
        fun("imu_ori_gyro_y", imu_ori_gyro_y);
        fun("imu_ori_gyro_z", imu_ori_gyro_z);
        fun("timestamp_s", timestamp_s);
        fun("timestamp_ns", timestamp_ns);
        fun("crc8", crc8);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("imu_ori_gyro_x", imu_ori_gyro_x);
        fun("imu_ori_gyro_y", imu_ori_gyro_y);
        fun("imu_ori_gyro_z", imu_ori_gyro_z);
        fun("timestamp_s", timestamp_s);
        fun("timestamp_ns", timestamp_ns);
        fun("crc8", crc8);
    }

    friend bool operator==(const ::physical_imu_origin_gyro_t& lhs, const ::physical_imu_origin_gyro_t& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.imu_ori_gyro_x - rhs.imu_ori_gyro_x)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.imu_ori_gyro_y - rhs.imu_ori_gyro_y)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.imu_ori_gyro_z - rhs.imu_ori_gyro_z)) < DBL_EPSILON) && (lhs.timestamp_s == rhs.timestamp_s) && (lhs.timestamp_ns == rhs.timestamp_ns) && (lhs.crc8 == rhs.crc8);
    }
};


#endif // IMPL_TYPE_PHYSICAL_IMU_ORIGIN_GYRO_T_H
