/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef IMPL_TYPE_PHYSICAL_VCU_INFO_1_T_H
#define IMPL_TYPE_PHYSICAL_VCU_INFO_1_T_H

#include <cfloat>
#include <cmath>
#include "impl_type_double.h"
#include "impl_type_uint8.h"
#include "impl_type_uint64.h"

struct physical_vcu_info_1_t {
    ::Double current_angle{ 0.0 };
    ::UInt8 ses_en_sts{ 0U };
    ::UInt8 motor_en_sts{ 0U };
    ::Double current_speed{ 0.0 };
    ::Double motor_torque{ 0.0 };
    ::UInt8 vcu_mode{ 0U };
    ::UInt8 controller_online_sts{ 0U };
    ::UInt8 ipc_online_status{ 0U };
    ::UInt8 vcu_life_signal{ 0U };
    ::Double current_brk_press{ 0.0 };
    ::UInt8 rseb_mode{ 0U };
    ::UInt8 battery_number{ 0U };
    ::UInt8 current_gear{ 0U };
    ::UInt8 battery_warning{ 0U };
    ::UInt8 current_spd_mode{ 0U };
    ::UInt64 timestamp_s{ 0U };
    ::UInt64 timestamp_ns{ 0U };
    ::UInt8 crc8{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(current_angle);
        fun(ses_en_sts);
        fun(motor_en_sts);
        fun(current_speed);
        fun(motor_torque);
        fun(vcu_mode);
        fun(controller_online_sts);
        fun(ipc_online_status);
        fun(vcu_life_signal);
        fun(current_brk_press);
        fun(rseb_mode);
        fun(battery_number);
        fun(current_gear);
        fun(battery_warning);
        fun(current_spd_mode);
        fun(timestamp_s);
        fun(timestamp_ns);
        fun(crc8);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(current_angle);
        fun(ses_en_sts);
        fun(motor_en_sts);
        fun(current_speed);
        fun(motor_torque);
        fun(vcu_mode);
        fun(controller_online_sts);
        fun(ipc_online_status);
        fun(vcu_life_signal);
        fun(current_brk_press);
        fun(rseb_mode);
        fun(battery_number);
        fun(current_gear);
        fun(battery_warning);
        fun(current_spd_mode);
        fun(timestamp_s);
        fun(timestamp_ns);
        fun(crc8);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("current_angle", current_angle);
        fun("ses_en_sts", ses_en_sts);
        fun("motor_en_sts", motor_en_sts);
        fun("current_speed", current_speed);
        fun("motor_torque", motor_torque);
        fun("vcu_mode", vcu_mode);
        fun("controller_online_sts", controller_online_sts);
        fun("ipc_online_status", ipc_online_status);
        fun("vcu_life_signal", vcu_life_signal);
        fun("current_brk_press", current_brk_press);
        fun("rseb_mode", rseb_mode);
        fun("battery_number", battery_number);
        fun("current_gear", current_gear);
        fun("battery_warning", battery_warning);
        fun("current_spd_mode", current_spd_mode);
        fun("timestamp_s", timestamp_s);
        fun("timestamp_ns", timestamp_ns);
        fun("crc8", crc8);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("current_angle", current_angle);
        fun("ses_en_sts", ses_en_sts);
        fun("motor_en_sts", motor_en_sts);
        fun("current_speed", current_speed);
        fun("motor_torque", motor_torque);
        fun("vcu_mode", vcu_mode);
        fun("controller_online_sts", controller_online_sts);
        fun("ipc_online_status", ipc_online_status);
        fun("vcu_life_signal", vcu_life_signal);
        fun("current_brk_press", current_brk_press);
        fun("rseb_mode", rseb_mode);
        fun("battery_number", battery_number);
        fun("current_gear", current_gear);
        fun("battery_warning", battery_warning);
        fun("current_spd_mode", current_spd_mode);
        fun("timestamp_s", timestamp_s);
        fun("timestamp_ns", timestamp_ns);
        fun("crc8", crc8);
    }

    friend bool operator==(const ::physical_vcu_info_1_t& lhs, const ::physical_vcu_info_1_t& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.current_angle - rhs.current_angle)) < DBL_EPSILON) && (lhs.ses_en_sts == rhs.ses_en_sts) && (lhs.motor_en_sts == rhs.motor_en_sts) && (fabs(static_cast<double>(lhs.current_speed - rhs.current_speed)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.motor_torque - rhs.motor_torque)) < DBL_EPSILON) && (lhs.vcu_mode == rhs.vcu_mode) && (lhs.controller_online_sts == rhs.controller_online_sts) && (lhs.ipc_online_status == rhs.ipc_online_status) && (lhs.vcu_life_signal == rhs.vcu_life_signal) && (fabs(static_cast<double>(lhs.current_brk_press - rhs.current_brk_press)) < DBL_EPSILON) && (lhs.rseb_mode == rhs.rseb_mode) && (lhs.battery_number == rhs.battery_number) && (lhs.current_gear == rhs.current_gear) && (lhs.battery_warning == rhs.battery_warning) && (lhs.current_spd_mode == rhs.current_spd_mode) && (lhs.timestamp_s == rhs.timestamp_s) && (lhs.timestamp_ns == rhs.timestamp_ns) && (lhs.crc8 == rhs.crc8);
    }
};


#endif // IMPL_TYPE_PHYSICAL_VCU_INFO_1_T_H
