/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_DMSERVICE_DMSERVICE_PROXY_H
#define JAC_AP_DMSERVICE_DMSERVICE_PROXY_H

#include "ara/com/internal/proxy/proxy_adapter.h"
#include "ara/com/internal/proxy/event_adapter.h"
#include "ara/com/internal/proxy/field_adapter.h"
#include "ara/com/internal/proxy/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/dmservice/dmservice_common.h"
#include <string>

namespace jac {
namespace ap {
namespace dmservice {
namespace proxy {
namespace events {
    using Decision4SMEvent = ara::com::internal::proxy::event::EventAdapter<::dpc::Decision4SM>;
    using ObjectTrimEvent = ara::com::internal::proxy::event::EventAdapter<::dpc::ObjectTrim>;
    using Decision4LongEvent = ara::com::internal::proxy::event::EventAdapter<::dpc::Decision4Long>;
    using Decision4LatEvent = ara::com::internal::proxy::event::EventAdapter<::dpc::Decision4Lat>;
    static constexpr ara::com::internal::EntityId DMServiceDecision4SMEventId = 44931U; //Decision4SMEvent_event_hash
    static constexpr ara::com::internal::EntityId DMServiceObjectTrimEventId = 57390U; //ObjectTrimEvent_event_hash
    static constexpr ara::com::internal::EntityId DMServiceDecision4LongEventId = 22274U; //Decision4LongEvent_event_hash
    static constexpr ara::com::internal::EntityId DMServiceDecision4LatEventId = 4690U; //Decision4LatEvent_event_hash
}

namespace fields {
}

namespace methods {

} // namespace methods

class DMServiceProxy {
private:
    std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
public:
    using HandleType = ara::com::HandleType;
    class ConstructionToken {
    public:
        explicit ConstructionToken(const std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> & proxy)
            : proxyAdapter(proxy),
              Decision4SMEvent(proxyAdapter->GetProxy(), events::DMServiceDecision4SMEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              ObjectTrimEvent(proxyAdapter->GetProxy(), events::DMServiceObjectTrimEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              Decision4LongEvent(proxyAdapter->GetProxy(), events::DMServiceDecision4LongEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              Decision4LatEvent(proxyAdapter->GetProxy(), events::DMServiceDecision4LatEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

        ConstructionToken(ConstructionToken&& other) = default;
        ConstructionToken& operator=(ConstructionToken && other) = default;

        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        
        virtual ~ConstructionToken() = default;

        /* Do not use such internal interfaces!!! */
        ara::core::Result<void> InitializeAll()
        {
            ara::core::Result<void> initResult;
            return initResult;
        }
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> GetProxyAdapter()
        {
            return proxyAdapter;
        }
        events::Decision4SMEvent GetDecision4SMEvent() noexcept { return std::move(Decision4SMEvent); }
        events::ObjectTrimEvent GetObjectTrimEvent() noexcept { return std::move(ObjectTrimEvent); }
        events::Decision4LongEvent GetDecision4LongEvent() noexcept { return std::move(Decision4LongEvent); }
        events::Decision4LatEvent GetDecision4LatEvent() noexcept { return std::move(Decision4LatEvent); }
    private:
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
        events::Decision4SMEvent Decision4SMEvent;
        events::ObjectTrimEvent ObjectTrimEvent;
        events::Decision4LongEvent Decision4LongEvent;
        events::Decision4LatEvent Decision4LatEvent;
    };

    virtual ~DMServiceProxy()
    {
        Decision4SMEvent.UnsetReceiveHandler();
        Decision4SMEvent.Unsubscribe();
        ObjectTrimEvent.UnsetReceiveHandler();
        ObjectTrimEvent.Unsubscribe();
        Decision4LongEvent.UnsetReceiveHandler();
        Decision4LongEvent.Unsubscribe();
        Decision4LatEvent.UnsetReceiveHandler();
        Decision4LatEvent.Unsubscribe();

    }

    explicit DMServiceProxy(const HandleType &handle)
        : proxyAdapter(std::make_shared<ara::com::internal::proxy::ProxyAdapter>(::jac::ap::dmservice::DMService::ServiceIdentifier, handle)),
          Decision4SMEvent(proxyAdapter->GetProxy(), events::DMServiceDecision4SMEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          ObjectTrimEvent(proxyAdapter->GetProxy(), events::DMServiceObjectTrimEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          Decision4LongEvent(proxyAdapter->GetProxy(), events::DMServiceDecision4LongEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          Decision4LatEvent(proxyAdapter->GetProxy(), events::DMServiceDecision4LatEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

    void ThrowError(const ara::core::Result<void>& result) const
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create proxy failed!"<< std::endl;
#endif
        }
    }

    DMServiceProxy(const DMServiceProxy&) = delete;
    DMServiceProxy& operator=(const DMServiceProxy&) = delete;

    DMServiceProxy(DMServiceProxy&&) = default;
    DMServiceProxy& operator=(DMServiceProxy&&) = default;
    DMServiceProxy(ConstructionToken&& token) noexcept
        : proxyAdapter(token.GetProxyAdapter()),
          Decision4SMEvent(token.GetDecision4SMEvent()),
          ObjectTrimEvent(token.GetObjectTrimEvent()),
          Decision4LongEvent(token.GetDecision4LongEvent()),
          Decision4LatEvent(token.GetDecision4LatEvent()){
        static_cast<void>(std::move(token));
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        const ara::com::HandleType &handle)
    {
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> preProxyAdapter =
            std::make_shared<ara::com::internal::proxy::ProxyAdapter>(
               ::jac::ap::dmservice::DMService::ServiceIdentifier, handle);
        ConstructionToken token{std::move(preProxyAdapter)};
        ara::core::Result<void> initResult {token.InitializeAll()};
        if (initResult.HasValue()) {
            return ara::core::Result<ConstructionToken>(std::move(token));
        } else {
            const ara::core::ErrorCode errorcode{initResult.Error()};
            ara::core::Result<DMServiceProxy::ConstructionToken> preResult{errorcode};
            return preResult;
        }
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType>& handler,
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::dmservice::DMService::ServiceIdentifier, instance);
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType> handler,
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::dmservice::DMService::ServiceIdentifier, specifier);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::dmservice::DMService::ServiceIdentifier, instance);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::dmservice::DMService::ServiceIdentifier, specifier);
    }

    static void StopFindService(const ara::com::FindServiceHandle& handle)
    {
        ara::com::internal::proxy::ProxyAdapter::StopFindService(handle);
    }

    HandleType GetHandle() const
    {
        return proxyAdapter->GetHandle();
    }
    bool SetEventThreadNumber(const std::uint16_t number, const std::uint16_t queueSize)
    {
        return proxyAdapter->SetEventThreadNumber(number, queueSize);
    }
    events::Decision4SMEvent Decision4SMEvent;
    events::ObjectTrimEvent ObjectTrimEvent;
    events::Decision4LongEvent Decision4LongEvent;
    events::Decision4LatEvent Decision4LatEvent;
};
} // namespace proxy
} // namespace dmservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_DMSERVICE_DMSERVICE_PROXY_H
